/*!
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
import { ScriptManager } from "./scriptManager";
export class WebCodeLoader {
    constructor(codeResolver, allowList) {
        this.codeResolver = codeResolver;
        this.allowList = allowList;
        this.loadedModules = new Map();
        this.scriptManager = new ScriptManager();
    }
    async seedModule(source, maybeFluidModule) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined
            && this.loadedModules.has(resolved.resolvedPackageCacheId)) {
            return;
        }
        const fluidModule = (maybeFluidModule !== null && maybeFluidModule !== void 0 ? maybeFluidModule : this.load(source));
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModule);
        }
    }
    async preCache(source, tryPreload) {
        var _a, _b, _c, _d, _e;
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (((_e = (_d = (_c = (_b = (_a = resolved) === null || _a === void 0 ? void 0 : _a.resolvedPackage) === null || _b === void 0 ? void 0 : _b.fluid) === null || _c === void 0 ? void 0 : _c.browser) === null || _d === void 0 ? void 0 : _d.umd) === null || _e === void 0 ? void 0 : _e.files) !== undefined) {
            return this.scriptManager.preCacheFiles(resolved.resolvedPackage.fluid.browser.umd.files, tryPreload);
        }
    }
    /**
     * @param source - Details of where to find chaincode
     */
    async load(source) {
        const resolved = await this.codeResolver.resolveCodeDetails(source);
        if (resolved.resolvedPackageCacheId !== undefined) {
            const maybePkg = this.loadedModules.get(resolved.resolvedPackageCacheId);
            if (maybePkg !== undefined) {
                return maybePkg;
            }
        }
        const fluidModuleP = this.loadModuleFromResolvedCodeDetails(resolved);
        if (resolved.resolvedPackageCacheId !== undefined) {
            this.loadedModules.set(resolved.resolvedPackageCacheId, fluidModuleP);
        }
        return fluidModuleP;
    }
    async loadModuleFromResolvedCodeDetails(resolved) {
        var _a;
        if (this.allowList !== undefined && !(await this.allowList.testSource(resolved))) {
            throw new Error("Attempted to load invalid code package url");
        }
        const loadedScripts = await this.scriptManager.loadLibrary(resolved.resolvedPackage.fluid.browser.umd);
        let fluidModule;
        for (const script of loadedScripts) {
            const maybeFluidModule = script.entryPoint;
            if (maybeFluidModule.fluidExport !== undefined) {
                if (fluidModule !== undefined) {
                    throw new Error("Multiple Fluid modules loaded");
                }
                fluidModule = maybeFluidModule;
            }
        }
        if (((_a = fluidModule) === null || _a === void 0 ? void 0 : _a.fluidExport) === undefined) {
            throw new Error("Entry point of loaded code package not a Fluid module");
        }
        return fluidModule;
    }
}
//# sourceMappingURL=webLoader.js.map